# -*- coding: utf-8 -*-

import argparse
import logging

import tfutility


class AbstractCommand:
    """
    An abstract class definition for custom CLI Commands
    """

    def __init__(
        self, parent: "tfutility.main.TfUtility", parser: argparse.ArgumentParser
    ):
        """Serves a abstract class for Commands. Used as Base for every controller

        :param parent: The TfUtility main Class which manages the CLI
        :type parent: TfUtility
        :param parser: The argparser Object
        :type parser: argparse.ArgumentParser
        """
        self._cmd = parent
        self.parser = parser

    def _init(self):
        """
        Intialize an subparser for this command
        normaly it has the name of this command
        """
        raise NotImplementedError("Please Implement an _init method")

    def get_logger(self) -> logging.Logger:
        """
        Get the Logger from the main class

        :return: The logger from the main class
        :rtype: logging.Logger
        """
        return self._cmd.get_logger()

    def add_arguments(self, parser: argparse.ArgumentParser):
        """Attach arguments to parser

        :param parser: The command parser
        :type parser: argparse.ArgumentParser
        :raises NotImplementedError: Triggerd when this method is not ovewritten
        """
        raise NotImplementedError("Please implement an add_arguments method")

    def get_command_name(self):
        """Return the command_name

        :raises NotImplementedError: Triggerd when this method is not ovewritten
        :return: The name of this command
        :rtype: str
        """
        if self.__class__.command_name is None:
            raise NotImplementedError("Please give your Command a name arg ")
        return self.command_name

    def get_help(self):
        if self.__class__.help is None:
            raise NotImplementedError("Please give your Command a help arg")
        return self.help

    def itsme(self, options: argparse.Namespace) -> bool:
        if options.command == self.get_command_name():
            return True
        return False

    def handle(self, option):
        raise NotImplementedError("Please implement this method in {self.__class__}")



# -*- coding: utf-8 -*-
import argparse

import pytest

from tfutility.core.abstract import AbstractCommand


def test_abstractcmd_constructor():
    x = AbstractCommand("a", "b")
    assert isinstance(x, AbstractCommand)
    assert x._cmd == "a"
    assert x.parser == "b"


def test_init():
    x = AbstractCommand("a", "b")
    with pytest.raises(NotImplementedError, match="Please Implement"):
        x._init()


def test_logger():
    class Sample:
        def get_logger(self):
            return "TEST"

    x = AbstractCommand(Sample(), "b")

    assert x.get_logger() == "TEST"


def test_add_arguments():
    x = AbstractCommand("a", "b")
    with pytest.raises(NotImplementedError, match=r"Please implement.*"):
        x.add_arguments(None)


def test_get_command_name():
    class NoneCommandName(AbstractCommand):
        command_name = None

    class DefinedCommandName(AbstractCommand):
        command_name = "commandname"

    x = NoneCommandName("a", "b")
    with pytest.raises(NotImplementedError, match=r"Please give your.*"):
        x.get_command_name()

    y = DefinedCommandName("a", "b")
    assert y.get_command_name() == "commandname"


def test_help():
    class NoneHelpText(AbstractCommand):
        help = "helptext"

    class DefinedHelpText(AbstractCommand):
        help = None

    x = DefinedHelpText("a", "b")
    with pytest.raises(NotImplementedError, match=r"Please give your.*"):
        x.get_help()

    y = NoneHelpText("a", "b")
    assert y.get_help() == "helptext"


def test_itsme():
    class TestClass(AbstractCommand):
        command_name = "commandfoo"
        help = "helptext"

    assert not TestClass(None, None).itsme(argparse.Namespace(command=None))
    assert not TestClass(None, None).itsme(argparse.Namespace(command="hiii"))

    assert TestClass(None, None).itsme(argparse.Namespace(command="commandfoo"))


def test_handle():
    class TestClass(AbstractCommand):
        command_name = "commandfoo"
        help = "helptext"

    with pytest.raises(NotImplementedError, match=r"Please implement this*"):
        assert not TestClass(None, None).handle(argparse.Namespace())



# -*- coding: utf-8 -*-
import re

from tfutility.main import main

VESION_REGEX = re.compile(r"^(\d+\.)?(\d+\.)?(\*|\d+)$")


def test_empty(capsys, mocker):
    mocker.patch("sys.argv", ["tfutility"])
    tfdo = mocker.patch("tfutility.main.TfUtility.do")
    main()
    captured = capsys.readouterr()
    assert captured.out == ""
    assert tfdo.call_count == 1


def test_version(capsys, mocker):
    mocker.patch("sys.argv", ["tfutility", "-V"])
    sysexit = mocker.patch("sys.exit")
    main()
    captured = capsys.readouterr()
    assert VESION_REGEX.match(captured.out) is not None
    assert sysexit.call_count == 1
